# Copyright (c) Jupyter Development Team.
# Distributed under the terms of the Modified BSD License.
#

from traittypes import Dataset
from traitlets import Unicode, Bool, Dict, Float, List

from .leaflet import Layer
from .xarray_ds import ds_x_to_json


class Velocity(Layer):
    """Velocity class, with Layer as parent class

    To visualize the direction and intensity of arbitrary velocities

    Attributes
    ----------
    data: dataset, default Empty dataset
        Underlying dataset
    zonal_speed: string, default ''
        Variable name in underlying dataset for the zonal speed.
    meridional_speed: string, default ''
       Variable name in underlying dataset for the meridional speed.
    latitude_dimension: string, default Empty dataset.
        Name of the latitude dimension in underlying dataset.
    longitude_dimension: string, default True
        Name of the longitude dimension in underlying dataset.
    units: string, default None
        Whether to show imperial units.
    display_values: bool, default True
        Display velocity data on mouse hover.
    display_options: dict, default {}
        Display options.
    min_velocity: float, default 0
        Used to align color scale
    max_velocity: float, default 10.0
        Used to align color scale.
    velocity_scale: float, 0.005
        To be modified for particle animations.
    color_scale: array, default []
        Array of hex/rgb colors for user-specified color scale.
    """

    _view_name = Unicode('LeafletVelocityView').tag(sync=True)
    _model_name = Unicode('LeafletVelocityModel').tag(sync=True)

    zonal_speed = Unicode('', help='Name of the zonal speed in the dataset')
    meridional_speed = Unicode('', help='Name of the meridional speed in the dataset')
    latitude_dimension = Unicode('latitude', help='Name of the latitude dimension in the dataset')
    longitude_dimension = Unicode('longitude', help='Name of the longitude dimension in the dataset')
    units = Unicode(None, allow_none=True)

    data = Dataset().tag(dtype=None, sync=True, to_json=ds_x_to_json)

    # Options
    display_values = Bool(True).tag(sync=True, o=True)
    display_options = Dict({
        'velocityType': 'Global Wind',
        'position': 'bottomleft',
        'emptyString': 'No velocity data',
        'angleConvention': 'bearingCW',
        'displayPosition': 'bottomleft',
        'displayEmptyString': 'No velocity data',
        'speedUnit': 'kt'
    }).tag(sync=True)
    min_velocity = Float(0).tag(sync=True, o=True)
    max_velocity = Float(10).tag(sync=True, o=True)
    velocity_scale = Float(0.005).tag(sync=True, o=True)
    color_scale = List([
        "rgb(36,104, 180)",
        "rgb(60,157, 194)",
        "rgb(128,205,193)",
        "rgb(151,218,168)",
        "rgb(198,231,181)",
        "rgb(238,247,217)",
        "rgb(255,238,159)",
        "rgb(252,217,125)",
        "rgb(255,182,100)",
        "rgb(252,150,75)",
        "rgb(250,112,52)",
        "rgb(245,64,32)",
        "rgb(237,45,28)",
        "rgb(220,24,32)",
        "rgb(180,0,35)"
    ]).tag(sync=True, o=True)



# Copyright (c) Jupyter Development Team.
# Distributed under the terms of the Modified BSD License.
#

"""
This module defines all the basemaps. These basemaps are XYZ tile providers coming from:
https://github.com/geopandas/xyzservices
:class:`xyzservices.providers`
Examples:

>>> from ipyleaflet.basemaps import basemaps
>>> basemaps.OpenStreetMap.Mapnik
"""
import xyzservices.providers as basemaps  # noqa



# Copyright (c) Jupyter Development Team.
# Distributed under the terms of the Modified BSD License.
#


def ds_x_to_json(ds, widget):
    return ds2json(
        ds,
        widget.zonal_speed,
        widget.meridional_speed,
        widget.latitude_dimension,
        widget.longitude_dimension,
        widget.units
    )


def ds2json(ds, u_var, v_var, lat_dim='latitude', lon_dim='longitude', units=None):
    """
    Assumes that the velocity components are given on a regular grid
    (fixed spacing in latitude and longitude).

    Parameters
    ----------
    u_var : str
        Name of the U-component (zonal) variable.
    v_var : str
        Name of the V-component (meridional) variable.
    lat_dim : str, optional
        Name of the latitude dimension/coordinate
        (default: 'latitude').
    lon_dim : str, optional
        Name of the longitude dimension/coordinate
        (default: 'longitude').
    units : str, optional
        Velocity units (default: try getting units from the
        'units' attributes of `u_var` and `v_var`).
    """
    import numpy as np
    ds = ds.copy()
    for var_name in (u_var, v_var):
        var_dims = ds[var_name].dims

        if set(var_dims) != set([lat_dim, lon_dim]):
            raise ValueError(
                "Invalid dimensions for variable '{}' in Dataset: "
                "should include only {}, found {}."
                .format(var_name, (lat_dim, lon_dim), var_dims)
            )

        # If dataset contains nans replace with 0
        ds[var_name] = ds[var_name].fillna(0)

    if units is None:
        u_var_units = ds[u_var].attrs.get('units')
        v_var_units = ds[v_var].attrs.get('units')

        if u_var_units != v_var_units:
            raise ValueError(
                "Different units found for U-component '{}' and "
                "V-component '{}' variables: '{}' and '{}'"
                .format(u_var, v_var, u_var_units, v_var_units))

        units = u_var_units

    if units is None:
        units = ''

    # Data should be in gaussian grid format (latitudes descending)
    if np.any(np.diff(ds[lat_dim].values) >= 0):
        ds = ds.sel(**{lat_dim: slice(None, None, -1)})

    # infer grid specifications (assume a rectangular grid)
    lat = ds[lat_dim].values
    lon = ds[lon_dim].values

    lon_left = float(lon.min())
    lon_right = float(lon.max())
    lat_lower = float(lat.min())
    lat_upper = float(lat.max())

    dx = float((lon_right - lon_left) / (lon.size - 1))
    dy = float((lat_upper - lat_lower) / (lat.size - 1))

    nx = lon.size
    ny = lat.size

    u_v_spec = ([2, 3],
                ["Eastward current", "Northward current"],
                [u_var, v_var])

    velocity_data = []

    for p_number, p_name, var_name in zip(*u_v_spec):
        velocity_data.append({
            "header": {
                "parameterUnit": units,
                "parameterNumber": p_number,
                "dx": dx, "dy": dy,
                "parameterNumberName": p_name,
                "la1": lat_upper,
                "la2": lat_lower,
                "parameterCategory": 2,
                "lo2": lon_right,
                "nx": nx,
                "ny": ny,
                "refTime": "2017-02-01 23:00:00",
                "lo1": lon_left
            },
            "data": ds[var_name].values.flatten().tolist()
        })

    return velocity_data



# Copyright (c) Jupyter Development Team.
# Distributed under the terms of the Modified BSD License.
#

version_info = (0, 17, 4)

__version__ = '%s.%s.%s' % (version_info[0], version_info[1], version_info[2])

EXTENSION_VERSION = '^0.17'



# Copyright (c) Jupyter Development Team.
# Distributed under the terms of the Modified BSD License.
#

import sys
from ._version import version_info, __version__  # noqa

# Allow dependencies to ipyleaflet to not be installed upon post-link for
# conda-build.

try:
    from .leaflet import *  # noqa
    from .basemaps import basemaps   # noqa

    if "google.colab" in sys.modules:
        from google.colab import output

        output.enable_custom_widget_manager()
except ImportError:
    pass


def _jupyter_labextension_paths():
    return [{
        'src': 'labextension',
        'dest': 'jupyter-leaflet'
    }]


def _jupyter_nbextension_paths():
    return [{
        'section': 'notebook',
        'src': 'nbextension',
        'dest': 'jupyter-leaflet',
        'require': 'jupyter-leaflet/extension'
    }]
